from DisplayTarget import DisplayTarget
from utils.datatypes import *

import gtk
import os


# whether to use image caching
_USE_CACHE = 1


#
# Class for image targets.
#
class TargetImage(DisplayTarget):

    def __init__(self, parent, display):

        # the pixbuf of the original image and its URI
        self.__original_image = None
        self.__uri = ""

        # the saturation of the image
        self.__saturation = 1.0

        # the size values
        self.__width = 0
        self.__height = 0
        self.__scale = 1.0
        
        # image cache
        self.__cache = {}
        

        DisplayTarget.__init__(self, parent, display)

        self.__widget = gtk.Image()
        self.__widget.show()

        self.add(self.__widget)

        self._set_property_type("uri", TYPE_STRING)
        self._set_property_type("saturation", TYPE_FLOAT)
        self._set_property_type("scale", TYPE_FLOAT)
        self._set_property_type("image-width", TYPE_INT)
        self._set_property_type("image-height", TYPE_INT)



    #
    # Sets the configuration values.
    #
    def set_config(self, key, value):

        value = self._convert_type(key, value)


        if (key == "uri"): self.__load_image(value)
        elif (key == "scale"): self.__scale_image(value)

        elif (key == "image-width"):
            self.__width = value
            self.__render_image(self.__width, self.__height)
            
        elif (key == "image-height"):
            self.__height = value
            self.__render_image(self.__width, self.__height)

        elif (key == "saturation"):
            self.__saturation = value
            #self.__render_image(self.__width, self.__height)

        else:
            DisplayTarget.set_config(self, key, value)




    #
    # Loads the image from the given URI.
    # TODO: use GNOME VFS
    #
    def __load_image(self, uri):

        uri = os.path.join(self.get_path(), uri)
        if (uri != self.__uri):
            self.__original_image = gtk.gdk.pixbuf_new_from_file(uri)
            self.__widget.set_from_pixbuf(self.__original_image)
            self.__uri = uri
            self.__cache = {}
        #end if

        if (self.__original_image):
            if (not self.__width):
                self.__width= self.__original_image.get_width()
            if (not self.__height):
                self.__height = self.__original_image.get_height()
            self.__render_image()
        #end if



    #
    # Scales the image by the given factor.
    #
    def __scale_image(self, scale):

        width = int(self.__width * scale)
        height = int(self.__height * scale)
        self.__render_image(width, height)
            
        self.__scale = scale



    #
    # Renders the image at the given size. Whithout size arguments,
    # the original size is used.
    #
    def __render_image(self, width = 0, height = 0):

        if (not self.__original_image): return

        w = self.__width #self.__original_image.get_width()
        h = self.__height #self.__original_image.get_height()

        if ((w, h) != (width, height) and width != 0 and height != 0):
            key = (width, height)
            if (_USE_CACHE and self.__cache.has_key(key)):
                img = self.__cache[key]
            else:
                img = self.__original_image.scale_simple(width, height, 3)
                if (_USE_CACHE): self.__cache[key] = img
                
        else:
            #img = self.__original_image
            img = self.__original_image.scale_simple(self.__width,
                                                     self.__height, 3)

        if (abs(self.__saturation - 1.0) > 0.00001):
            simg = img.copy()
            img.saturate_and_pixelate(simg, self.__saturation, gtk.FALSE)
        else:
            simg = img

        self.__widget.set_from_pixbuf(simg)
